(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

open Parsetree;;

(* test functions on the type relation *)

val is_rewrite : relation -> bool;;
val is_commutative : relation -> bool;;
val is_associative : relation -> bool;;
val is_absorbent : relation -> bool;;
val is_absorbing : relation -> bool;;
val is_neutral : relation -> bool;;
val is_idempotent : relation -> bool;;
val is_inverse : relation -> bool;;
val is_distributive : relation -> bool;;

(* test functions on lists of relations *)

val has_commutative : relations -> bool;;
val has_associative : relations -> bool;;
val has_idempotent : relations -> bool;;
val has_absorbing : relations -> bool;;

(* access function on lists of relations (first from left to right) *)

val rewrite : relations -> (pattern * expression option * pattern) list;;
val commutative : relations -> relation_side option;;
val associative : relations -> relation_side option;;
val absorbent : relations -> (relation_side * string) option;;
val absorbing : relations -> (relation_side * string) option;;
val neutral : relations -> (relation_side * string) option;;
val idempotent : relations -> relation_side option;;
val inverse : relations -> (relation_side * string * string option) option;;
val distributive : relations -> (relation_side * string) list;;

(* Name of a given relation (getting rid of the arguments of the relation. *)

val name_of_relation : relation -> string;;

(* Total ordering for relations based on precedences of their various
   associated rewrite rules. *)

val compare_relation : relation -> relation -> int;;

module RelOrd : Set.OrderedType;;

module RelSet : Set.S with type elt = relation;;
