(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: genr_vary.ml,v 1.11 2007/04/25 15:04:50 weis Exp $ *)

(* We treat the case of unary constructors whose unique argument
 is in fact the variable length list of arguments of an associative
 binary operation. *)

open Relation;;
open Spec;;
open Code;;
open Parsetree;;

open Genr_base;;

let genr_mem e =
  Let ("mem", ["x"],
    Letrec ("aux", ["l"],
      Match (Id "l", [
        Clause (Id "[]", Id "false");
        Clause (appi (Id "y") "::" (Id "m"),
          Match (app2 "Pervasives.compare" (Id "x") (Id "y"), [
            Clause (Id "0", Id "true");
            Clause_when (Id "k", appi (Id "k") "<" (Id "0"), Id "false");
            Clause (underscore, app1 "aux" (Id "m"))
	  ])
        )
      ]),
    Id "aux"),
    e);;

let genr_unique_args e =
  genr_mem
    (Let ("unique_args", [],
      Letrec ("aux", ["acc"; "l"],
        Match (Id "l", [
          Clause (Id "[]", Id "acc");
          Clause (appi (Id "x") "::" (Id "m"),
            If (app2 "mem" (Id "x") (Id "acc"),
              (app2 "aux" (Id "acc") (Id "m")),
              (app2 "aux" (appi (Id "x") "::" (Id "acc")) (Id "m"))
	    )
	  )
        ]),
        app1 "aux" (Id "[]")),
     e));;

let genr_flatten_args cname e =
  Let ("flatten_args", ["t"; "m"],
       Match (Id "t", [
	      Clause (pattern cname [Id "l"],
                app2 "List.append" (Id "l") (Id "m"));
	      Clause (underscore, appi (Id "t") "::" (Id "m"))
	    ]),
       e);;

let genr_distrib_args =
  let arg = appi (Id "acc") "@" (appi (Id "y") "::" (Id "rest")) in
  fun cname dname e ->
  Letrec ("distrib_args", ["acc"; "l"],
    Match (Id "l", [
      Clause
        (appi (pattern dname [Id "m"]) "::" (Id "rest"),
         let0 "acc" (app1 "List.rev" (Id "acc"))
          (let1 "dist" "y" (constr cname [arg])
           (constr dname [app2 "List.map" (Id "dist") (Id "m")])));
      Clause
        (appi (Id "t") "::" (Id "rest"),
         app2 "distrib_args" (appi (Id "t") "::" (Id "acc")) (Id "rest"));
      Clause (underscore, app1 "raise" (Id "Not_found"))
      ]),
    e);;

let genr_fun_1_assoc_by_ap fdef arg ap e =
  match ap with
  | Associative _s ->
      genr_flatten_args fdef.fd_uname
        (let0 arg
           (app3 "List.fold_right" (Id "flatten_args") (Id arg) (Id "[]"))
           e)
  | Absorbent (_s, dname) ->
      If (app2 "List.mem" (Id dname) (Id arg),
          Id dname,
          e)
  | Neutral (_s, dname) ->
      let0 arg
        (app2 "List.filter" (app1 "(<>)" (Id dname)) (Id arg))
        (Match (Id arg, [
               Clause (Id "[]", Id dname);
               Clause (Id "[x]", Id "x");
               Clause (underscore, e)
             ]))
  | Distributive (_s, dname) ->
      genr_distrib_args fdef.fd_uname dname
      (Try (app2 "distrib_args" (Id "[]") (Id arg),
           [Clause (Id "Not_found", e)]))
  | Commutative _s ->
      let0 arg
        (app2 "List.sort" (Id "compare") (Id arg))
        e
  | Idempotent _s ->
      genr_unique_args (let0 arg (app1 "unique_args" (Id arg)) e)

  (* FIXME: to be completed ! *)
  | _ -> assert false;;

let genr_fun_1_assoc fdef arg =
  List.fold_right (genr_fun_1_assoc_by_ap fdef arg) fdef.fd_relations
    (constr fdef.fd_uname [Id arg]);;
