(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2007,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: listutil.ml,v 1.4 2007/04/26 10:58:05 blanqui Exp $ *)

(* Eliminates duplicates in a list. *)

let unique l =
  let rec uniq acc = function
    | x :: l -> if List.mem x acc then uniq acc l else uniq (x :: acc) l
    | _ -> acc in
  uniq [] l;;

(* Membership in a list sorted in increasing order. *)

let mem_sorted cmp x =
  let rec mem = function
    | [] -> false
    | y :: l ->
	match cmp x y with
	  | 0 -> true
	  | k when k < 0 -> false
	  | _ -> mem l
  in mem;;

(* Eliminates duplicates in a list sorted in increasing order.
Result list in decreasing order. *)

let unique_sorted l =
  let rec uniq acc = function
    | x :: (y :: m as l) -> if x = y then uniq acc l else uniq (x :: acc) l
    | [x] -> x :: acc
    | _ -> acc in
  uniq [] l;;

(* Provides the x's of Some(x)'s obtained by f. *)

let rec filter_map_opt f = function 
  | x :: l ->
    (match f x with
     | Some y -> y :: filter_map_opt f l
     | _ -> filter_map_opt f l)
  | _ -> [];;
